/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.undo;

import org.w3c.dom.Document;

import ch.cern.lhcb.xmleditor.Editor;

/**
 * this is an extension of the default CompoundEdit that handles some
 * application specific tasks
 */

public class CompoundEdit extends javax.swing.undo.CompoundEdit implements
        DocumentUndoableEdit {

    /**
     * the editor using this CompoundEdit
     */
    protected Editor editor;

    /**
     * default constructor
     * 
     * @param editor the editor using this CompoundEdit
     */
    public CompoundEdit(Editor editor) {
        super();
        this.editor = editor;
    }

    /**
     * Sets inProgress to false. Overridden here to update the editor menus
     * accordingly.
     */
    public void end() {
        super.end();
        if (editor != null) {
            editor.updateEditMenu();
        }
    }

    /**
     * Send conditionnalDieAndTrim to each subedit, in the reverse of the order
     * that they were added. Removes the edits that died. Returns true if the
     * compound itself died, false if not.
     * 
     * @param document a document
     * @return true if it died, false if not
     */
    public boolean conditionnalDieAndTrim(Document document) {
        int size = edits.size();
        for (int i = size - 1; i >= 0; i--) {
            try {
                DocumentUndoableEdit e = (DocumentUndoableEdit) edits
                        .elementAt(i);
                if (e.conditionnalDieAndTrim(document)) {
                    edits.removeElementAt(i);
                }
            } catch (java.lang.ClassCastException e) {
                // nothing to do, this edit was not a DocumentUndoableEdit
            }
        }
        if (edits.size() == 0) {
            super.die();
            editor.updateEditMenu();
            return true;
        }
        editor.updateEditMenu();
        return false;
    }

}