/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.cese;

import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Vector;

import javax.swing.tree.TreePath;

import com.javable.cese.templates.AbstractDataExport;
import com.simulogic.dataview.DataChannel;
import com.simulogic.dataview.DataFrame;
import com.simulogic.dataview.DataGroup;
import com.simulogic.dataview.TimeChannel;

/**
 * This class exports data from model to selected window and controls certain
 * variables in <code>clamp</code> mode.
 * 
 * @see com.javable.cese.templates.AbstractDataExport
 */
public class DataExport extends AbstractDataExport {

	/**
	 * Specify Track experiment
	 */
	public static final int TRACK_EXPERIMENT = 0;

	/**
	 * Specify command experiment
	 */
	public static final int CLAMP_EXPERIMENT = 1;

	private Method getters[];

	private DataChannel channels[];

	private RecordBuilder builder;

	private DataFrame frame;

	private int expType;

	/**
	 * Prepares export to the top/new <tt>Results view</tt> window based on
	 * current <tt>Export</tt> selection
	 */
	public void prepareExport() {
		expType = CESE.getEnvironment().getModelManager().getExpType();
		builder = CESE.getEnvironment().getModelManager().getRecordBuilder();

		if ((model.getsteps() <= 0) || (model.getdt() <= 0))
			throw new IllegalArgumentException();

		createChannels();

		if (expType == CLAMP_EXPERIMENT)
			builder.buildRecord(model);
	}

	/**
	 * Exports data from the model to the top/new <tt>Results view</tt> window
	 * 
	 * @param count
	 *            Current iteration
	 */
	public void doExport(int count) {
		if (expType == CLAMP_EXPERIMENT)
			builder.clampProperties(count);

		try {
			for (int i = 0; i < channels.length; i++) {
				channels[i].setData(count, ((Double) getters[i].invoke(model,
						(Object[]) null)).doubleValue());
			}
		} catch (Exception e) {
		}
	}

	/**
	 * Finishes export to the top/new <tt>Results view</tt> window
	 */
	public void finishExport() {
		if (channels.length > 0) {
			try {
				java.awt.EventQueue.invokeAndWait(new Runnable() {
					public void run() {
						frame = CESE.getEnvironment().getDesktop().addFrame(
								CESE.getEnvironment().getModelManager()
										.getModelInfo().getName());
					}
				});
			} catch (InterruptedException e) {
				Thread.currentThread().interrupt();
			} catch (InvocationTargetException e) {
			}

			if (frame != null) {
				int gp = frame.getDataContent().getStorage().getGroupsSize();

				String mname = CESE.getEnvironment().getModelManager()
						.getModelInfo().getName();
				DataGroup group = new DataGroup(ResourceManager
						.getResource("Sweep")
						+ " " + gp + " (" + mname + ")", mname);

				group.addChannel(new TimeChannel(model.getdt(), model
						.getsteps()));
				for (int i = 0; i < channels.length; i++) {
					group.addChannel(channels[i]);
				}

				frame.finishLoad(group);
			}
		}
	}

	private void createChannels() {
		Object expvar[];
		TreePath exptree[];

		Vector ch = new Vector();
		Vector ex = new Vector();
		PropertyDescriptor pd;

		try {
			exptree = CESE.getEnvironment().getModelManager()
					.getExportTreeSelection();

			for (int i = 0; i < exptree.length; i++) {
				expvar = exptree[i].getPath();
				for (int j = 0; j < expvar.length; j++) {
					if (expvar[j] instanceof PropertyDescriptor) {
						pd = (PropertyDescriptor) expvar[j];
						if (pd.getReadMethod() != null) {
							if (pd.getReadMethod().invoke(model,
									(Object[]) null) instanceof Number) {
								ex.addElement(pd.getReadMethod());
								ch.addElement(new DataChannel(pd.getName()
										.substring(5), model.getsteps()));
							}
						}
					}
				}
			}
		} catch (Exception e) {
		}
		int exp = ex.size();
		channels = new DataChannel[exp];
		getters = new Method[exp];
		for (int i = 0; i < exp; i++) {
			channels[i] = (DataChannel) ch.get(i);
			getters[i] = (Method) ex.get(i);
		}
	}

	/**
	 * Getter for property builder.
	 * 
	 * @return Value of property builder.
	 */
	public RecordBuilder getBuilder() {
		return this.builder;
	}

}