/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview;

import javax.swing.tree.DefaultMutableTreeNode;

/**
 * Class that represents group of data sets (channels) from a single recording
 */
public class DataGroup extends DefaultMutableTreeNode {

    private String name;
    private String info;

    private int xChannel;

    /**
     * Creates new DataGroup
     * 
     * @param name name of the group
     * @param info description
     */
    public DataGroup(String name, String info) {
        super();
        this.name = name;
        this.info = info;
    }

    /**
     * Adds a channel to this group
     * 
     * @param c channel to add
     */
    public void addChannel(DataChannel c) {
        this.add(new DefaultMutableTreeNode(c));
    }

    /**
     * Returns channel in the group
     * 
     * @return DataChannel for index
     * @param c channel index
     * @throws ClassCastException if object is not of type DataChannel
     */
    public DataChannel getChannel(int c) throws ClassCastException {
        return (DataChannel) ((DefaultMutableTreeNode) this.getChildAt(c))
                .getUserObject();
    }

    /**
     * Getter for property name
     * 
     * @return Value of property name
     */
    public String getName() {
        return name;
    }

    /**
     * Setter for property name
     * 
     * @param name New value of property name
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Getter for property info
     * 
     * @return Value of property info
     */
    public String getInfo() {
        return info;
    }

    /**
     * Setter for property info
     * 
     * @param info New value of property info
     */
    public void setInfo(String info) {
        this.info = info;
    }

    /**
     * Getter for property xChannel.
     * 
     * @return Value of property xChannel.
     */
    public int getXChannel() {
        return xChannel;
    }

    /**
     * Setter for property xChannel.
     * 
     * @param x New value of property xChannel.
     */
    public void setXChannel(int x) {
        this.xChannel = x;
    }
}